﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace OxyplotTest
{
    public class PlotData : INotifyPropertyChanged
    {

        #region Property Changed

        public event PropertyChangedEventHandler PropertyChanged;

        protected void RaisePropertyChanged([CallerMemberName]string propertyName = null)
        {
            this.VerifyPropertyName(propertyName);

            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// Warns the developer if this object does not have a public property with the specified name. 
        /// This method does not exist in a Release build.
        /// </summary>
        [Conditional("DEBUG")]
        [DebuggerStepThrough]
        private void VerifyPropertyName(string propertyName)
        {
            // Verify that the property name matches a real, public, instance property on this object.
            if (TypeDescriptor.GetProperties(this)[propertyName] == null)
            {
                Debug.Fail(String.Format("Invalid property name: {0}", propertyName));
            }
        }

        #endregion

        public PlotData()
        {
            //Initialize OxyPlot Graph
            this.PlotModel = new OxyPlot.PlotModel { Title = "" };

            //Axes
            this.PlotModel.Axes.Add(new OxyPlot.Axes.LinearAxis());
            this.PlotModel.Axes.Add(new OxyPlot.Axes.LinearAxis());

            this.PlotModel.Axes[0].Position = OxyPlot.Axes.AxisPosition.Bottom;
            //this.PlotModel.Axes[0].Minimum = 0.0;
            this.PlotModel.Axes[0].Title = "X Axis";
            this.PlotModel.Axes[0].MajorGridlineColor = OxyPlot.OxyColors.Black;
            this.PlotModel.Axes[0].MajorGridlineStyle = OxyPlot.LineStyle.Solid;

            this.PlotModel.Axes[1].Position = OxyPlot.Axes.AxisPosition.Left;
            //this.PlotModel.Axes[1].Minimum = 0.0;
            this.PlotModel.Axes[1].Title = "Y Axis";
            this.PlotModel.Axes[1].MajorGridlineColor = OxyPlot.OxyColors.Black;
            this.PlotModel.Axes[1].MajorGridlineStyle = OxyPlot.LineStyle.Solid;

            this.PlotModel.IsLegendVisible = true;
        }
        
        private OxyPlot.PlotModel plotModel;

        public OxyPlot.PlotModel PlotModel
        {
            get
            {
                return this.plotModel;
            }
            set
            {

                this.plotModel = value;
                RaisePropertyChanged();
            }
        }

        public void AddSeries()
        {
            //Create OxyPlot lineSeries
            //OxyPlot.Series.LineSeries lineSeries = new OxyPlot.Series.LineSeries();

            //Random r = new Random();

            //for (int i = 0; i < 40; i++)
            //{
            //    //First Point
            //    OxyPlot.DataPoint dataPoint = new OxyPlot.DataPoint(r.NextDouble() * 100, r.Next(100));
            //    lineSeries.Points.Add(dataPoint);
            //}

            //lineSeries.MarkerType = OxyPlot.MarkerType.Circle;

            //this.PlotModel.Series.Add(lineSeries);

            //this.PlotModel.InvalidatePlot(true);

            List<Func<double, double>> functions = new List<Func<double, double>>
            {
                Math.Cos,
                Math.Sin,
                Math.Tan,
                Math.Cosh,
                Math.Sinh,
                Math.Tanh
            };

            List<string> labels = new List<string>
            {
                "cos(x)",
                "sin(x)",
                "tan(x)",
                "cosh(x)",
                "sinh(x)",
                "tanh(x)"
            };

            Random r = new Random();

            int index = r.Next(0, 5);

            this.PlotModel.Series.Add(new OxyPlot.Series.FunctionSeries(functions[index], 0, 10, 0.1, labels[index]));

            //Specify a refresh so binding knows that model has changed
            this.PlotModel.InvalidatePlot(true);
        }
    }
}
